/* Copyright (C) 2000-2018 Hiroshi Watabe watabe@cyric.tohoku.ac.jp */
/* mullin.c created from mullin.f using f2c
$Id: mullin.c,v 2fcc4ac67d16 2018/05/14 22:42:12 watabe $
$Date: 2004-03-08 09:45:33 +0900 (月, 08  3月 2004) $
*/
#include "bld.h"
#define ABS(x) ((x) >= 0 ? (x) : -(x))

/*  MULLIN.FOR - ROUTINE TO PERFORM MULTIPLE LINEAR REGRESSION */

/*  Last Edit     :  25-jan-90 */
/*  Application   :  BLD */
/*  Author        :  RE CARSON */
/*  Modifications : */
/*  ------------------------------------------------------------------------ */
/* 25-jan-90  store corrmat at end of procedure */
/*  23-mar-1989 : normalized all independent variables to norm 1 */
/* 		then correct estimates at end */
/*  27-DEC-85 : P. COLLARD : Edited for VAX/VMS */
/*  ------------------------------------------------------------------------ */

/*  MODULES IN THIS FILE:  MULLIN */

/*  MULLIN(N,P,indep_x,Y,PARA,SE,WT,WEIGHT,FIT,SS,SECU,B,work) - THIS SUBROUTINE WILL */
/* 	PRODUCE A LEAST-SQUARES FIT (ORDINARY OR WEIGHTED) OF ONE DEPENDENT */
/* 	VARIABLE VERSUS 'P' INDEPENDENT VARIABLES */

/*  ARGUMENTS: */

/* 	N	NUMBER OF POINTS(INTEGER) */
/* 	P	NUMBER OF DEPENDENT VARIABLES(INTEGER) */
/* 	POS(P)	POSITIONS OF DEP. VARIABLES IN COMMON ARRAY X */
/*       indepx(p*n) independent values array by hiroshi */
/* 	Y(N)	DEPENDENT VARIABLE */
/* 	PARA(P)	PARAMETER ESTIMATES (OUTPUT) */
/* 	SE(P)	STANDARD ERRORS OF ESTIMATES (OUTPUT) */
/* 	WT(N)	WEIGHTING VECTPR */
/* 	WEIGHT	LOGICAL*1  TRUE IF WEIGHTING IS TO BE DONE */
/* 	FIT(N)	FITTED CURVE (OUTPUT) */
/* 	SS	RESIDUAL SUM OF SQUARES (OUTPUT) */
/* 	SECU	STANDARD ERROR ABOUT CURVE (OUTPUT) */
/* 	B(P,P)	WORK VARIABLE(REAL*8) */
/* 	work(p) work array (real*8) */

/*    SS IS RETURNED AS -1 IF THE DETERMINANT IS 0 */
/* 	N	= integer*4 */
/* 	P	= integer*4 */
/* 	POS	= real*4 array */
/* 	Y	= real*4 array */
/* 	PARA	= real*4 array */
/* 	SE	= real*4 array */
/* 	WT	= real*4 array */
/* 	WEIGHT	= logical*1 */
/* 	FIT	= real*4 array */
/* 	SS	= real*8 */
/* 	SECU	= real*8 */
/* 	B	= real*8 array */
/* 	work	= real*8 array */

/* - */
/*        SUBROUTINE MULLIN(N,P,indepx,Y,PARA,SE,WT,WEIGHT,FIT,SS, */
/*      1    SECU,B,work,cormat) */
int mullin_(int nindep,int npos,float *indepx,float *y,float *para,double *se,float *wt,char *weight, double *fit,double *ss, double *secu,double *bmat,double *work,float *cormat)
{
  /* Initialized data */
  static double cut = 1e-20;

  /* System generated locals */
  int cormat_dim1, b_dim1;
  double d1;

  /* Local variables */
  static double temp, temp2;
  static int i, j, k;
  static double rnorm;
  static double det;

  /* Parameter adjustments */
  cormat_dim1 = npos;
  b_dim1 = npos;

/*  THIS ROUTINE COMPUTES PARA=INV(AT * WT * A) * AT * WT * Y */
/*    WHERE AT = A TRANSPOSE */

/*  COMPUTE B = AT * WT * A */

/* DET=0 CUTOFF */
  for (i = 0; i < npos; ++i) {
    for (j = 0; j < npos; ++j) {
      temp = (float)0.;
      for (k = 0; k < nindep; ++k) {
	temp2 = (double) indepx[nindep*i + k] * (double)indepx[nindep*j + k];
	if (*weight) {
	  temp2 *= wt[k];
	}
	temp += temp2;
      }
      bmat[i + j*b_dim1] = temp;
    }
  }

/*  calculate the norm of each independent variable - store in para(i) */

    for (i = 0; i < npos; ++i) {
	rnorm = (float)0.;
	for (k = 0; k < nindep; ++k) {
	    rnorm += (double) indepx[nindep*i + k] * (double)indepx[nindep*i + k];
	}
	work[i] = sqrt(rnorm);
	if (work[i] == 0.) {
	  *ss = -1.;
	  return 1;
	}
    }

/*  divide out the norm of each indep variable from rows and columns */
/* 	of B so its determinant approaches 1 */
    for (i = 0; i<npos; ++i) {
	for (j = 0; j<npos; ++j) {
	    bmat[i + j*b_dim1] = bmat[i + j*b_dim1] / work[i] / work[j];
	}
    }

/*  COMPUTE B = INV(B) */

    det = 0.;
    dminv_(bmat, npos, &det, fit, se);
    if (ABS(det) < cut) {
      *ss = -1.;
      return 1;
    }

/*  divide out the norm of each indep variable from rows and columns */
/* 	of Binverse so its value is recovered */

    for (i = 0; i < npos; ++i) {
      for (j = 0; j < npos; ++j) {
	bmat[i + j*b_dim1] = bmat[i + j*b_dim1] / work[i]/work[j];
      }
    }

/*  COMPUTE   WORK VECTOR = AT * WT * Y - STORE IN SE */

    for (i = 0; i < npos; ++i) {
	temp = (float)0.;
	for (k = 0; k < nindep; ++k) {
	    temp2 = (double) indepx[nindep * i + k] * (double) y[k];
	    if (*weight) {
		temp2 *= wt[k];
	    }
	    temp += temp2;
	}
	work[i] = temp;
    }

/*  COMPUTE PARA = B * WORK VECTOR - ACCOUNT FOR SCALING DURING INVERSE */

    for (i = 0; i < npos; ++i) {
	temp = (float)0.;
	for (k = 0; k < npos; ++k) {
	    temp += bmat[i + k*b_dim1]*work[k];
	}
	para[i] = (float)temp;
    }

/*  COMPUTE FITTED CURVE AND SUM OF SQUARES */

    *ss = (float)0.;
    for (k = 0; k < nindep; ++k) {
      temp = (float)0.;
      for (i = 0; i < npos; ++i) {
	temp += (double) indepx[nindep * i + k] * (double)para[i];
      }
      fit[k] = temp;
      /* Computing 2nd power */
      d1 = temp - y[k];
      temp = d1 * d1;
      if (*weight) {
	temp *= wt[k];
      }
      *ss += temp;
    }

/*  COMPUTE STANDARD ERRORS */

    *secu = sqrt(*ss/(nindep - npos));
    for (i = 0; i < npos; ++i) {
	se[i] = sqrt(bmat[i + i*b_dim1]) * *secu;
    }

/*  CALCULATE CORRELATION MATRIX AND STORE IN A VARIABLE CALLED CORRMAT */
    for (i = 0; i < npos; ++i) {
      for (j = 0; j < npos; ++j) {
	cormat[i + j*cormat_dim1] = (float)(bmat[i + j*b_dim1] / sqrt(bmat[i + i*b_dim1] * bmat[j + j*b_dim1]));
/* CALCULATE CORRELATION */
      }
    }

    return 0;
} /* mullin_ */

